@@ -1,11 +1,11 @@
-import asyncio
 from .errors import WrongType
-from .tools import MISSING, _or, components_to_dict, setup_logger
+from .tools import MISSING, components_to_dict, setup_logger
 
 import discord
 from discord.http import Route
 
 import json
+import asyncio
 from typing import List
 
 logging = setup_logger(__name__)
@@ -38,7 +38,7 @@ async def send_files(route, files, payload, http):
 
     return await http.request(route, form=form, files=files)
 
-def jsonifyMessage(content=MISSING, tts=False, embed: discord.Embed=MISSING, embeds: List[discord.Embed]=MISSING, attachments: List[discord.Attachment]=MISSING, nonce: int=MISSING,
+def get_message_payload(content=MISSING, tts=False, embed: discord.Embed=MISSING, embeds: List[discord.Embed]=MISSING, attachments: List[discord.Attachment]=MISSING, nonce: int=MISSING,
                 allowed_mentions: discord.AllowedMentions=MISSING, reference: discord.MessageReference=MISSING, mention_author: bool=MISSING, components: list=MISSING, stickers: List[discord.Sticker]=MISSING, suppress: bool=MISSING, flags=MISSING):
     """Turns parameters from send functions into a payload for requests"""
     
@@ -50,21 +50,24 @@ def jsonifyMessage(content=MISSING, tts=False, embed: discord.Embed=MISSING, emb
         else:
             payload["content"] = str(content)
 
-    if suppress is not MISSING:
-        flags = discord.MessageFlags._from_value(_or(flags, discord.MessageFlags.DEFAULT_VALUE))
+    if suppress not in [MISSING, None]:
+        flags = discord.MessageFlags._from_value(flags or discord.MessageFlags.DEFAULT_VALUE)
         flags.suppress_embeds = suppress
         payload['flags'] = flags.value
     
-    if nonce is not MISSING:
+    if nonce not in [MISSING, None]:
         payload["nonce"] = nonce
     
     if embed is not MISSING or embeds is not MISSING:
-        if embed is not MISSING and embeds is MISSING:
-            embeds = [embed]
-        elif embed is not MISSING and embeds is not MISSING:
-            embeds.append(embed)
-        if embeds is None or embed is None:
+        if embeds is None and embed is None:
             embeds = []
+        # if embed exsists and embeds doesn't exsist
+        elif embed not in [MISSING, None] and embeds in [MISSING, None]:
+            embeds = [embed]
+        # if embed doesn't exsist and embeds does
+        elif embed in [MISSING, None] and embeds not in [MISSING, None]:
+            embeds = embed
+        # check type things
         elif not all(isinstance(x, discord.Embed) for x in embeds):
             raise WrongType("embeds", embeds, 'list[discord.Embed]')
         payload["embeds"] = [em.to_dict() for em in embeds]
@@ -77,7 +80,7 @@ def jsonifyMessage(content=MISSING, tts=False, embed: discord.Embed=MISSING, emb
                 raise WrongType("attachments", attachments, "List[discord.attachment]")
             payload["attachments"] = [x.to_dict() for x in attachments]
 
-    if reference is not MISSING:
+    if reference is not MISSING and reference is not None:
         if not isinstance(reference, (discord.MessageReference, discord.Message)):
             raise WrongType("reference", reference, ['discord.MessageReference', 'discord.Message'])
         if isinstance(reference, discord.MessageReference):
@@ -87,12 +90,12 @@ def jsonifyMessage(content=MISSING, tts=False, embed: discord.Embed=MISSING, emb
 
     if allowed_mentions is not MISSING:
         if allowed_mentions is None:
-            payload["allowed_mentions"] = discord.AllowedMentions()
+            allowed_mentions = discord.AllowedMentions()
         else: 
             if not isinstance(allowed_mentions, discord.AllowedMentions):
                 raise WrongType("allowed_mentions", allowed_mentions, "discord.AllowedMentions")
         payload["allowed_mentions"] = allowed_mentions.to_dict()
-    if mention_author is not MISSING:
+    if mention_author is not MISSING and mention_author is not None:
         allowed_mentions = payload["allowed_mentions"] if "allowed_mentions" in payload else discord.AllowedMentions().to_dict()
         allowed_mentions['replied_user'] = mention_author
         payload["allowed_mentions"] = allowed_mentions
